using System;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization.Formatters.Soap;

[Serializable]
class Animal
{
	private string mDescription;
	[NonSerialized] private int mSpeed;

	public string Description
	{
		get
		{
			return mDescription;
		}
		set
		{
			mDescription = value;
		}
	}

	public int Speed
	{
		get
		{
			return mSpeed;
		}
		set
		{
			mSpeed = value;
		}
	}
}

class SerializeToFileDemo
{
	static void DoSerialization()
	{
		Animal animal1 = new Animal();
		animal1.Description = "One pretty chicken";
		animal1.Speed = 3;

		Animal animal2 = new Animal();
		animal2.Description = "Buggs bunny";
		animal2.Speed = 1000;
		
		IFormatter formatter = new BinaryFormatter();
		Stream stream = new FileStream("data.bin", FileMode.Create);
		using (stream)
		{
			formatter.Serialize(stream, animal1);
			formatter.Serialize(stream, animal2);
		}
	}

	static void DoDeserialization()
	{
		IFormatter formatter = new BinaryFormatter();
		Stream stream = new FileStream("data.bin", FileMode.Open);
		using (stream)
		{
			Animal animal1 = (Animal) formatter.Deserialize(stream);
			Console.WriteLine("(Description: {0}, Speed: {1})", 
				animal1.Description, animal1.Speed);
		
			Animal animal2 = (Animal) formatter.Deserialize(stream);
			Console.WriteLine("(Description: {0}, Speed: {1})", 
				animal2.Description, animal2.Speed);
		}
	}
		
	static void Main()
	{
		Console.WriteLine("Performing serialization.");
		DoSerialization();
		Console.WriteLine("Done.\n");

		Console.WriteLine("Performing deserialization.");
		DoDeserialization();
		Console.WriteLine("Done.\n");
	}
}
