using System;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Soap;

namespace Demo_5_ISerializable
{
	[Serializable]
	public class Person : ISerializable
	{
		protected int mAge;
		protected string mName;
		
		public Person(string aName, int aAge)
		{
			mName = aName;
			mAge = aAge;
		}

		protected Person(SerializationInfo aInfo, StreamingContext aContext)
		{
			mName = aInfo.GetString("Person's name");
			mAge = aInfo.GetInt32("Person's age");
		}

		public virtual void GetObjectData(SerializationInfo aInfo,
			StreamingContext aContext)
		{
			aInfo.AddValue("Person's name", mName);
			aInfo.AddValue("Person's age", mAge);
		}
	}

	[Serializable]
	sealed class Employee : Person
	{
		private string mJobPosition;

		public Employee(string aName, int aAge, string aJobPosition) 
			: base(aName, aAge)
		{
			mJobPosition = aJobPosition;
		}

		private Employee(SerializationInfo aInfo, StreamingContext aContext)
			: base(aInfo, aContext)
		{
			mJobPosition = aInfo.GetString("Employee's job");
		}
        
		public override void GetObjectData(SerializationInfo aInfo,
			StreamingContext aContext)
		{
			base.GetObjectData(aInfo, aContext);
			aInfo.AddValue("Employee's job", mJobPosition);
		}

		public override string ToString()
		{
			string value = String.Format("(Name: {0}, Age: {1}, Job: {2})",
				mName, mAge, mJobPosition);
			return value;
		}
	}

	class ISerializableDemo
	{
		static void Main()
		{
			Employee employee = new Employee("Jeffrey Richter", 45, "CEO");
			Console.WriteLine("Employee = {0}", employee);
			FileStream empoyeeFile = new FileStream("employee.xml", FileMode.Create);
			using (empoyeeFile)
			{
				IFormatter formatter = new SoapFormatter();
				formatter.Serialize(empoyeeFile, employee);
				Console.WriteLine("Employee serialized.");
				
				empoyeeFile.Seek(0, SeekOrigin.Begin);
				Employee deserializedEmployee = 
					(Employee) formatter.Deserialize(empoyeeFile);
				Console.WriteLine("Employee deserialized.");
				Console.WriteLine("Deserialized = {0}", deserializedEmployee);
			}
		}
	}
}
